#include "Wire.h"
#include <SPI.h> // Not actually used but needed to compile
#include <LowPower.h> //https://github.com/rocketscream/Low-Power
#include <SparkFunBME280.h> //https://github.com/sparkfun/SparkFun_BME280_Arduino_Library/tree/master/src
#include <SoftwareSerial.h>
#include <DS3232RTC.h> //https://github.com/JChristensen/DS3232RTC

#define DS3231_I2C_ADDRESS 0x68

BME280 mySensorA;
SoftwareSerial mySerial(7, 6); //SIM800L Tx & Rx is connected to Arduino #7 & #6
float temp,pressure,humid,v1,v2;
int tempint,pressureint,t1,t2,alarmminute,i,n,attempts;
int interval=30;
float voltage=0.747;
char data1[10],data2[10],data3[10],s[5],response[30],sendstring[120],out[50];

time_t p;
volatile byte wakecause;
bool conn=0;
bool restart=1;
byte sensorno=2; //If you have two sensors, change to the sensor you are writing to.
char apikey[2][20] = {"yourAPI-Key-1","yourAPI-Key-2"}; //replace with your own ThingSpeak Read API keys for sensors 1 and 2


byte decToBcd(byte val) {return( (val/10*16) + (val%10) );}
byte bcdToDec(byte val) {return( (val/16*10) + (val%16) ); }

void setup()
{
v1=0; v2=0;
startsensor(); delay(50);
mySerial.begin(4800);
Serial.begin(9600);    // Debugging only
pinMode(2,INPUT_PULLUP); //RTC Alarm interrupt
pinMode(3,INPUT_PULLUP); //Push button
pinMode(4,OUTPUT); //Voltage regulator enable
pinMode(5,OUTPUT); //BME280 enable via MOSFET
pinMode(6,OUTPUT); //Tx to SIM800
pinMode(13,OUTPUT);
analogReference(INTERNAL);
pinMode(A0,INPUT);
digitalWrite(5,HIGH);
digitalWrite(4,LOW);


Serial.println("Starting BME280s... result of .begin():");
Serial.print("Sensor A: 0x");
Wire.begin();
RTC.setAlarm(ALM1_EVERY_SECOND,45,0,0,0);
RTC.setAlarm(ALM2_MATCH_DATE, 0, 0, 0, 1);
RTC.alarm(ALARM_1);
RTC.alarm(ALARM_2);
RTC.alarmInterrupt(ALARM_1, false);
RTC.alarmInterrupt(ALARM_2, false);
RTC.squareWave(SQWAVE_NONE);
RTC.setBBSQW(1);

RTC.alarm(ALARM_1);
// configure the INT/SQW pin for "interrupt" operation (disable square wave output)
RTC.squareWave(SQWAVE_NONE);
// enable interrupt output for Alarm 1
RTC.alarmInterrupt(ALARM_1, true);
attachInterrupt(0,donothing1,FALLING);  
attachInterrupt(1,donothing2,FALLING);  
}

void loop()
{
  Serial.println("Start loop");
  Serial.println("Millis = " + (String)millis());
  p=RTC.get();
  byte second, minute, hour, dayOfWeek, dayOfMonth, month, year;
  Serial.println("Loop1");
  readDS3231time(&second, &minute, &hour, &dayOfWeek, &dayOfMonth, &month,
  &year);
  
  Serial.println("Loop2");
  alarmminute=((int((minute)/interval)+1)*interval)%60;
  Serial.println("Set alarm to minute: "+(String)alarmminute);
  RTC.setAlarm(ALM1_MATCH_MINUTES,0,alarmminute,0,0);
  displayTime(); 
  attachInterrupt(0,donothing1,FALLING);  
  digitalWrite(5,0); digitalWrite(6,0); digitalWrite(7,0);
  RTC.alarm(ALARM_1);
  RTC.alarm(ALARM_2);
  Serial.println("Go to sleep"); delay(150);
  LowPower.powerDown(SLEEP_FOREVER, ADC_OFF, BOD_OFF); 
  detachInterrupt(0);
  digitalWrite(5,1);
  Serial.println("Wake up");
  RTC.alarm(ALARM_1);
  Serial.println("Cause: "+(String)(wakecause==0 ? "Timer":"Button"));
  Serial.println("Timer signal: "+(String)(digitalRead(2)==0 ? "Low":"High"));
  // if ((wakecause==0 && restart==0) || wakecause==1) {Serial.println("Do Read and Send"); readandsend();  }
  Serial.println("Raw A0: "+(String)analogRead(A0));
  v1=readvoltage()*voltage;
  Serial.println("Voltage at wakeup: "+(String)v1);
  {Serial.println("Do Read and Send"); readandsend();  }
  restart=0;
}


void readandsend()
{
  digitalWrite(4,HIGH);
  t1=millis();
  startsensor(); delay(10);
  temp=mySensorA.readTempC(); 
  humid=mySensorA.readFloatHumidity();
  pressure=mySensorA.readFloatPressure()/100; 
  dtostrf(temp,5,2,data1);
  dtostrf(pressure,7,2,data2);
  dtostrf(humid,4,1,data3);
  sprintf(out,"Temp: %s Pressure: %s Humidity %s",data1,data2,data3);
  Serial.println(out);
  Serial.println(apikey[1]);
  digitalWrite(13,1); delay(10); digitalWrite(13,0);
  Serial.println(v1);
  Serial.println(v2);
  Serial.println(sendstring);
  delay(150);
  delay(10000);
  Serial.println("Initializing...");
  delay(1000);
  mySerial.println("AT"); 
  mySerial.flush();
  t2=millis();
  attempts=0; conn=0;
  while (!conn && attempts<25)
    {attempts++;
    Serial.println("Start !conn loop");
    mySerial.println("AT+CREG?"); 
    while (!mySerial.available() && millis()-t2<5000) delay(5);
    i=0;
    while (mySerial.available()) {response[i]=mySerial.read(); i+=1; delay(2);}
    Serial.println("Last characters (n="+(String)i+") ");
//    for (n=0; n<i; n++) {Serial.print((char)response[n]) +(String)" "; Serial.print(response[n],DEC); Serial.println(" "+(String)n);} Serial.println(); 
    Serial.println("The 20th character was " + (String)(response[20]==49 ? "":"not ") + (String)"1"); 
    if (response[20]==49) conn=1;
    if (conn==0) delay(1000);
    }
  v2=readvoltage()*voltage;
  Serial.println("v2="+(String)v2);
  sprintf(sendstring,"AT+HTTPPARA=\"url\",\"http://api.thingspeak.com/update?api_key=%s&field1=%s&field2=%s&field3=%s&field4=%.3d%.3d\"",apikey[sensorno-1],data1,data2,data3,(int)v1,(int)v2);
  Serial.println("Voltage at load: "+(String)v2);
  if (conn==1) {
    Serial.print("This is the response: ");
    Serial.write(response,i);
    Serial.write(0x0d);
    delay(1000);
    updateSerial();
    mySerial.println("AT+CSQ"); //Signal quality test, value range is 0-31 , 31 is the best
    updateSerial();
    //mySerial.println("AT+CIPSHUT"); 
    mySerial.println("AT+CSTT=\"everywhere\",\"eesecure\",\"secure\"");
    updateSerial();
    mySerial.println("AT+CIICR");
    attempts=0; conn=0; t2=millis();
    while (!conn && attempts<10)
      {attempts++;
      Serial.println("Start CIICR loop");
      while (!mySerial.available() && millis()-t2<5000) delay(5);
      i=0;
      while (mySerial.available()) {response[i]=mySerial.read(); i+=1; delay(2);}
      Serial.println("Last characters (n="+(String)i+") ");
      for (n=0; n<i; n++) {Serial.print((char)response[n]) +(String)" "; Serial.print(response[n],DEC); Serial.println(" "+(String)n);} Serial.println(); 
      Serial.println("The 3rd character was " + (String)(response[2]==79 ? "":"not ") + (String)"O"); 
      if (i>1 && response[2]==79) conn=1;
      if (conn==0) delay(1000);
      }
    updateSerial();
    mySerial.println("AT+SAPBR=3,1,\"CONTYPE\",\"GPRS\""); 
    updateSerial();
    mySerial.println("AT+CIFSR"); 
    delay(1000);
    mySerial.println("AT+SAPBR=3,1,\"APN\",\"everywhere\""); 
    updateSerial();
    delay(1000);
    mySerial.println("AT+SAPBR=1,1"); 
    updateSerial();
    mySerial.println("at+httpinit");
    delay(1000);
    updateSerial();
    
//    mySerial.println(sendstring);
//    mySerial.println("AT+HTTPPARA=\"url\",\"http://api.thingspeak.com/update?api_key="+apikey[sensorno]+(String)data1+(String)"&field2="+(String)data2+(String)"&field3="+(String)data3+(String)"\"");
    mySerial.println(sendstring);
    updateSerial();
    delay(1000);
    mySerial.println("AT+HTTPACTION=0");
    delay(2000);
    updateSerial();
    delay(2000);
    updateSerial(); }
  
  
  digitalWrite(4,0); 
  delay(500);
  dtostrf((millis()-t1)/1000.0,5,1,s);
  Serial.println("Power on: "+(String)s);
}

void startsensor()
  {digitalWrite(5,HIGH); delay(5);
  Serial.println(mySensorA.begin(), HEX);
  mySensorA.settings.commInterface = I2C_MODE;
  mySensorA.settings.I2CAddress = 0x76;
  mySensorA.settings.runMode = 1; //  3, Normal mode 1,forced
  mySensorA.settings.tStandby = 0; //  0, 0.5ms
  mySensorA.settings.filter = 0; //  0, filter off
  mySensorA.settings.tempOverSample = 1;
  mySensorA.settings.pressOverSample = 1;
  mySensorA.settings.humidOverSample = 1;
  }

void updateSerial()
{
  delay(100);
  while (Serial.available()) {mySerial.write(Serial.read()); }
  while(mySerial.available()) {Serial.write(mySerial.read());}
}

void readDS3231time(byte *second, byte *minute, byte *hour, byte *dayOfWeek, byte *dayOfMonth, byte *month, byte *year)
{
  Wire.beginTransmission(DS3231_I2C_ADDRESS);
  Wire.write(0); 
  Wire.endTransmission();
  Wire.requestFrom(DS3231_I2C_ADDRESS, 7);
  *second = bcdToDec(Wire.read() & 0x7f);
  *minute = bcdToDec(Wire.read());
  *hour = bcdToDec(Wire.read() & 0x3f);
  *dayOfWeek = bcdToDec(Wire.read());
  *dayOfMonth = bcdToDec(Wire.read());
  *month = bcdToDec(Wire.read());
  *year = bcdToDec(Wire.read());
}
void displayTime()
{
  byte second, minute, hour, dayOfWeek, dayOfMonth, month, year;
  readDS3231time(&second, &minute, &hour, &dayOfWeek, &dayOfMonth, &month,
  &year);
  Serial.print(hour, DEC);
 Serial.print("h");
  if (minute<10) {Serial.print("0");}
  Serial.print(minute, DEC);
  Serial.print("min");
  if (second<10) {Serial.print("0");}
  Serial.print(second, DEC);
  Serial.print("sec  ");
  Serial.print(dayOfMonth, DEC);
  Serial.print("/");
  Serial.print(month, DEC);
  Serial.print("/");
  Serial.print(year, DEC);
  Serial.print(" ");
  Serial.print(" Day : ");
  switch(dayOfWeek){
  case 1:
    Serial.println("Sunday");
    break;
  case 2:
    Serial.println("Monday");
    break;
  case 3:
    Serial.println("Tuesday");
    break;
  case 4:
    Serial.println("Wednesday");
    break;
  case 5:
    Serial.println("Thursday");
    break;
  case 6:
    Serial.println("Friday");
    break;
  case 7:
    Serial.println("Saturday");
    break;
  }
}

void donothing1 () //Alarm
{Serial.println("Do nothing 0.");
wakecause=0; 
}

void donothing2 () //Button
{Serial.println("Do nothing 1.");
wakecause=1; 
}

float readvoltage()
{ int i=0;
  for (int n=0; n<3; n++) {delay(10); i+=analogRead(A0);}
  return i/3.0;
  }